/*
 * Project name:
     RTC3 click (Demonstration of mikroBUS RTC3 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150401:
     - initial release (DM);
 * Description:
      This project is simple demonstration how to read/write date and time from
      BQ32000DR RTC (real-time clock).
      Date and time are read from the RTC every 10 milisecond and printed on LCD.
      Pressing A0 button will write default time and date to RTC module.
      When button is released RTC will start counting.
      User can change default time and date in main function.
 * Test configuration:
      MCU:             PIC18F45K22
                       http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
      Dev. Board:      EasyPIC v7 Connectivity
                       http://www.mikroe.com/easypic/
      Oscillator:      HS-PLL, 8MHz Crystal, 32MHz MCU Clock
      ext. modules:    RTC3 click board click
                       http://www.mikroe.com/click/RTC3/
      SW:              mikroC PRO for PIC
                       http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Turn on LCD backlight switch SW4.6. (board specific)
     - Place Pull up on RA0 (port A three state switch), and J17 to GND position
     - RTC3 click board with BQ32000DR is placed to the mikroBUS socket 1.
       Program uses I2C1 perpheral to comunicate with RTC chip.
     - Place power selection jumper (J5) in 5V position.
 */
 
// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;

// Lcd constants
char txt1[] = "mikroElektronika";
char txt2[] = "RTC3 click";
char txt3[] = "DATE:";
char txt4[] = "TIME:";

// Module variables
unsigned char hours, minutes, seconds, day, week, month, year;    // Global date/time variables
char oldstate = 0;

// RTC Definitions
#define RTC_ADDR  0xD0

/**************************************************************************************************
* BQ32000DR Functions
**************************************************************************************************/

/**************************************************************************************************
* Read data from RTC BQ32000DR
* input : addres of RTC register
* output: value of of RTC register
**************************************************************************************************/
unsigned char RTC_Read(unsigned char addr){
  unsigned char value;

  I2C1_Start();                   // Issue start signal
  I2C1_Wr(RTC_ADDR);              // Address BQ32000DR, see BQ32000DR datasheet
  I2C1_Wr(addr);                  // Start from address 2
  I2C1_Start();                   // Issue repeated start signal
  I2C1_Wr(RTC_ADDR);              // Address BQ32000DR for reading R/W=1

  value = I2C1_Rd(0);             // Read seconds byte
  I2C1_Stop();                    // Issue stop signal
  
  return value;
}

/**************************************************************************************************
* Write data from BQ32000DR
* input : addres of RTC register, value of of RTC register
**************************************************************************************************/
void RTC_Write(unsigned char addr, unsigned char value){
  I2C1_Start();         // Issue start signal
  I2C1_Wr(RTC_ADDR);    // Address BQ32000DR
  I2C1_Wr(addr);        // Start from address
  I2C1_Wr(value);       // Write value to RTC register
  I2C1_Stop();          // Issue stop signal
}

/**************************************************************************************************
* Read time from RTC BQ32000DR
* input : pointer to variables where RTC data will be stored
* output: variables with RTC data
**************************************************************************************************/
void Read_Time(unsigned char *p_hours, unsigned char *p_minutes, unsigned char *p_seconds,
               unsigned char *p_day, unsigned char *p_week, unsigned char *p_month, unsigned char *p_year){
  I2C1_Start();                // Issue start signal
  I2C1_Wr(RTC_ADDR);           // Address BQ32000DR, see BQ32000DR datasheet
  I2C1_Wr(0);                  // Start from address 0
  I2C1_Repeated_Start();       // Issue repeated start signal
  I2C1_Wr(RTC_ADDR + 1);       // Address BQ32000DR for reading R/W=1

  *p_seconds = I2C1_Rd(1);     // Read seconds byte
  *p_minutes = I2C1_Rd(1);     // Read minutes byte
  *p_hours = I2C1_Rd(1);       // Read hours byte
  *p_week =I2C1_Rd(1);
  *p_day =I2C1_Rd(1);
  *p_month =I2C1_Rd(1);
  *p_year =I2C1_Rd(0);

  I2C1_Stop();                 // Issue stop signal
}

/**************************************************************************************************
* Write time to RTC BQ32000DR
* input : variables with RTC data
**************************************************************************************************/
void Write_Time(unsigned char c_hours, unsigned char c_minutes, unsigned char c_seconds,
                unsigned char c_day, unsigned char c_week, unsigned char c_month, unsigned char c_year){
   I2C1_Start();                     // issue start signal
   I2C1_Wr(RTC_ADDR);                // address BQ32000DR
   I2C1_Wr(0);                       // start from word at address (REG0)
   I2C1_Wr(0x80);                    // write $80 to REG0. (pause counter + 0 sec)

   I2C1_Wr(c_minutes);               // write 0 to minutes word to (REG1)
   I2C1_Wr(c_hours);                 // write 17 to hours word (24-hours mode)(REG2)
   I2C1_Wr(c_week);                  // write 2 - Monday (REG3)
   I2C1_Wr(c_day);                   // write 4 to date word (REG4)
   I2C1_Wr(c_month);                 // write 5 (May) to month word (REG5)
   I2C1_Wr(c_year);                  // write 01 to year word (REG6)
   I2C1_Stop();                      // issue stop signal

   I2C1_Start();                     // issue start signal
   I2C1_Wr(RTC_ADDR);                // address BQ32000DR
   I2C1_Wr(0);                       // start from word at address 0
   I2C1_Wr(0 | c_seconds);           // write 0 to REG0 (enable counting + 0 sec)
   I2C1_Stop();                      // issue stop signal
}

/**************************************************************************************************
* Show on the LCD display
* input : variables with RTC data
**************************************************************************************************/
void Show_Time(){
  char *txt;

  seconds  =  ((seconds & 0x70) >> 4)*10 + (seconds & 0x0F);
  minutes  =  ((minutes & 0x70) >> 4)*10 + (minutes & 0x0F);
  hours    =  ((hours & 0x30) >> 4)*10 + (hours & 0x0F);
  week     =  (week & 0x07);
  day      =  ((day & 0xF0) >> 4)*10 + (day & 0x0F);
  month    =  ((month & 0x10) >> 4)*10 + (month & 0x0F);
  year     =  ((year & 0xF0)>>4)*10+(year & 0x0F);

  switch(week)
  {
    case 1: txt="Sun"; break;
    case 2: txt="Mon"; break;
    case 3: txt="Tue"; break;
    case 4: txt="Wed"; break;
    case 5: txt="Thu"; break;
    case 6: txt="Fri"; break;
    case 7: txt="Sat"; break;
  }

  LCD_Out(1,1, txt);
  Lcd_Chr(1, 6, (day / 10)   + 48);    // Print tens digit of day variable
  Lcd_Chr(1, 7, (day % 10)   + 48);    // Print oness digit of day variable
  Lcd_Chr(1, 9, (month / 10) + 48);
  Lcd_Chr(1,10, (month % 10) + 48);
  Lcd_Chr(1,14, (year / 10)  + 48);
  Lcd_Chr(1,15, (year % 10)  + 48);

  Lcd_Chr(2, 6, (hours / 10)   + 48);
  Lcd_Chr(2, 7, (hours % 10)   + 48);
  Lcd_Chr(2, 9, (minutes / 10) + 48);
  Lcd_Chr(2,10, (minutes % 10) + 48);
  Lcd_Chr(2,12, (seconds / 10) + 48);
  Lcd_Chr(2,13, (seconds % 10) + 48);
}

/**************************************************************************************************
* MAIN PROGRAM
**************************************************************************************************/
void main(){
  ANSELA = 0;                        // Configure PORTA pins as digital
  ANSELB = 0;                        // Configure PORTB pins as digital
  ANSELC = 0;                        // Configure PORTC pins as digital
  ANSELD = 0;
  ANSELE = 0;                        // Configure PORTE pins as digital
  SLRCON = 0;                        // Configure all PORTS at the standard Slew

  // Configure user buttons
  TRISA0_bit = 1;                    // Set RA0 pin as input.
                                     // RA0 is used to write inital data to RTC.

  I2C1_Init(100000);                 // Initialize I2C bus for communication with RTC
  Lcd_Init();                        // Initialize Lcd
  
  // charging super - capacitor
  I2C1_Start();
  I2C1_Wr(RTC_ADDR);
  I2C1_Wr(0x08);
  I2C1_Wr(0x20);
  I2C1_Wr(0x45);
  I2C1_Stop;

  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
  Lcd_Out(1,1,txt1);                 // Write text in first row
  Lcd_Out(2,4,txt2);                 // Write text in second row

  Delay_ms(2000);                    // delay 2 sec

  Lcd_Cmd(_LCD_CLEAR);               // Prepare and output static text on LCD
  LCD_Chr(1,8,'.');
  LCD_Chr(1,11,'.');
  LCD_Out(2,1,"Time:");
  LCD_Chr(2,8,':');
  LCD_Chr(2,11,':');
  LCD_Out(1,12,"20");

  // Start the test examples
  while(1){                           // Endless loop
    if (Button(&PORTA, 0, 1, 0)) {               // Detect logical one
      oldstate = 1;                              // Update flag
    }
    if (oldstate && Button(&PORTA, 0, 1, 1)) {   // Detect one-to-zero transition
    // Write TIME    16  : 30  : 00  : 1.  Wednesday april  2015
      Write_Time(0x16, 0x30, 0x00, 0x01, 0x04,   0x04, 0x15);
      oldstate = 0;                              // Update flag
    }

    Read_Time(&hours, &minutes, &seconds, &day, &week, &month, &year);
    Show_Time();
    Delay_mS(10);
  }
}